import 'dart:async';
import 'package:flutter_plugin_android_lifecycle/flutter_plugin_android_lifecycle.dart';

import 'package:flutter/services.dart';
import 'package:uuid/uuid.dart';

class AmazonS3SDK {
  String bucketName;
  String region;
  String accessKey;
  String secretKey;

  String _uploaderKey;

  static const MethodChannel _channel = const MethodChannel('amazons3_sdk');

  static const UPLOAD_SUCCESS = true;
  static const UPLOAD_FAIL = false;

  static const UPLOADED_FILE_DEL_SUCCESS = true;
  static const UPLOADED_FILE_DEL_FAILED = false;

  static const UPLOADED_FILE_EXISTS = true;
  static const UPLOADED_FILE_ABSENT = false;

  static const NO_DATA = null;
  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  ///  METHODES PUBLIQUES
  ///
  ///
  /// constructeur
  AmazonS3SDK({
    this.bucketName,
    this.region,
    this.accessKey,
    this.secretKey,
  }) {
    _createUploaderKey();

    _initUploader(
      accessKey,
      secretKey,
      bucketName,
      region,
      _uploaderKey,
    );
  }

  /// File String String => Future<UploadFileResult>
  ///
  /// permet d'uploader un fichier.
  /// retourne un objet UploadFileResult contenant:
  ///
  /// {
  ///   "url": "http://lurldufichier.com",
  ///   "key": "le/path/vers/le/fichier/sans/extension",
  ///   "uploadSuccess": bool (si oui ou non l'upload est réussi)
  /// }
  Future<UploadFileResult> uploadFile({
    String filePath,
    String key,
    Function onBefore,
    Function(dynamic) onDone,
    Function(dynamic) onError,
  }) async {
    Map arguments = {
      "filePath": filePath,
      "key": key,
      "uploaderKey": _uploaderKey,
    };

    UploadFileResult resultFromDart;

    try {
      onBefore();

      Map resultFromPlatform = await _channel.invokeMethod(
        'uploadFile',
        arguments,
      );

      resultFromDart = UploadFileResult.fromMap(
        resultFromPlatform,
      );

      onDone(resultFromDart);

      return resultFromDart;
    } on PlatformException catch (e) {
      print(
        "Oups... une erreur est survenue lors de l'upload de fichier:\n\n${e.message}",
      );

      resultFromDart = _uploadFileBadResult();

      return resultFromDart;
    }
  }

  /// String => Future<bool>
  ///
  /// Prend comme input le path vers le fichier à supprimer
  /// (un path SANS l'extension du fichier)
  ///
  /// permet de supprimer un fichier uploadé.
  /// return si oui ou non la suppression à été effectuée avec succès
  /// (ATTENTION !! si le path spécifié
  /// pointe vers in fichier inexistant,
  /// vous allez qd meme recevoir true pour la deletion)
  Future<bool> deleteUploadedFile(String key) async {
    bool deletionOutcome;

    if (key == null) {
      return UPLOADED_FILE_DEL_SUCCESS;
    }

    Map arguments = {
      "key": key,
      "uploaderKey": _uploaderKey,
    };

    try {
      deletionOutcome = await _channel.invokeMethod(
        'deleteUploadedFile',
        arguments,
      );
    } on PlatformException catch (e) {
      deletionOutcome = UPLOADED_FILE_DEL_FAILED;

      print(
        "Oups... une erreur est survenue lors de la suppression de fichier:\n\n${e.message}",
      );
    }

    return deletionOutcome;
  }

  /// String => Future<bool>
  ///
  /// Prend comme input le path vers le fichier dans le bucket
  /// (un path SANS l'extension du fichier)
  ///
  /// permet de savoir si un fichier existe dans un bucket
  Future<bool> uploadedFileExists(String key) async {
    bool fileExists;

    Map arguments = {
      "key": key,
      "bucketName": bucketName,
      "uploaderKey": _uploaderKey,
    };

    try {
      if (key == null) {
        return true;
      }

      fileExists = await _channel.invokeMethod('uploadedFileExists', arguments);
    } on PlatformException catch (e) {
      fileExists = UPLOADED_FILE_ABSENT;

      print(
        "Oups... une erreur est survenue en vérifiant que le fichier uploadé existe:\n\n${e.message}",
      );
    }

    return fileExists;
  }

  Future<bool> deleteBucketFolder(String key) async {
    Map arguments;
    bool folderDeleted;

    if (key == NO_DATA) {
      return true;
    } else {
      arguments = {
        "key": key,
        "uploaderKey": _uploaderKey,
      };
    }

    try {
      folderDeleted = await _channel.invokeMethod(
        'deleteBucketFolder',
        arguments,
      );

      print("folder deleted ?: $folderDeleted");
    } on PlatformException catch (e) {
      print("Erreur durant la suppression de folder: ${e.message}");

      folderDeleted = false;
    }

    return folderDeleted;
  }

  ///  METHODES PRIVEES
  ///
  ///

  UploadFileResult _uploadFileBadResult() {
    return UploadFileResult(
      uploadSuccess: UPLOAD_FAIL,
    );
  }

  void _createUploaderKey() {
    Uuid uuidGenerator = Uuid();
    String uuid = uuidGenerator.v1();

    _uploaderKey = uuid;
  }

  /// String String String String String => Future
  ///
  /// crée un uploader pour un bucket particulier
  Future<void> _initUploader(
    String accessKey,
    String secretKey,
    String bucketName,
    String region,
    String uploaderKey,
  ) async {
    Map arguments = {
      "accessKey": accessKey,
      "secretKey": secretKey,
      "region": region,
      "bucketName": bucketName,
      "uploaderKey": uploaderKey,
    };

    try {
      int qtyOfUploaders = await _channel.invokeMethod(
        'initUploader',
        arguments,
      );

      print("**** How many uploaders exist right now: $qtyOfUploaders");
    } on PlatformException catch (e) {
      print(
        "Oups... une erreur est survenue lors de l'initialisation de l'uploader de fichiers:\n\n${e.message}",
      );
    }
  }
}

/// le resultat d'un upload de fichier
class UploadFileResult {
  bool uploadSuccess;
  String key;
  String url;

  UploadFileResult({
    this.uploadSuccess,
    this.key,
    this.url,
  });

  factory UploadFileResult.fromMap(Map data) {
    return UploadFileResult(
      uploadSuccess: data['uploadSuccess'],
      key: data['key'],
      url: data['url'],
    );
  }

  Map toMap() {
    return {
      'uploadSuccess': uploadSuccess,
      'key': key,
      'url': url,
    };
  }
}
